﻿using System;
using System.Windows.Forms;
using Cybele.Thinfinity;
using System.Data;

namespace fishfact3
{
    public partial class FormMain : Form
    {
        private IJSObject mRemoteDB = null;

        public FormMain() {
            InitializeComponent();
        }

        public IJSObject RemoteDB {
            get {
                if (mRemoteDB == null) {
                    // Creates the "db" JSObject
                    mRemoteDB = new JSObject("db");

                    var allowedTypes = new[] { typeof(String), typeof(Byte), typeof(Char), typeof(Decimal), typeof(Double) };

                    // Iterates over the dataset fields
                    foreach (DataColumn column in dbdemosDataSet.BIOLIFE.Columns) {
                        // Bypasses some data types
                        if (Array.IndexOf(allowedTypes, column.DataType) >= 0) {
                            // Creates the property with the name of the field.
                            mRemoteDB.Properties.Add(column.ColumnName)     // Returns a IJSProperty
                                .OnGet(new JSBinding(                       // Adds a "getter" to the returned IJSProperty
                                    // This anonymous procedure do the actual get
                                    delegate (IJSObject AObj, IJSProperty AProp) {
                                        DataColumn col = dbdemosDataSet.BIOLIFE.Columns[AProp.Name];
                                        string value = dbdemosDataSet.BIOLIFE.Rows[bIOLIFEBindingSource.Position][col].ToString();

                                        if ((col.DataType == typeof(String)) || (col.DataType == typeof(Byte)) || (col.DataType == typeof(Char))) {
                                            AProp.AsString = value;
                                        }
                                        else {
                                            AProp.AsFloat = float.Parse(value);
                                        }
                                    }
                                ));
                        }
                    }
                    // Applies the object model and sends it to the web browser
                    mRemoteDB.ApplyModel();
                }
                return mRemoteDB;
            }
        }

        private void FormMain_Load(object sender, EventArgs e) {
            // TODO: This line of code loads data into the 'dbdemosDataSet.BIOLIFE' table. You can move, or remove it, as needed.
            this.bIOLIFETableAdapter.Fill(this.dbdemosDataSet.BIOLIFE);

        }

        private void bIOLIFEBindingSource_PositionChanged(object sender, EventArgs e) {
            // Calls all mRemoteDB's properties "getters"
            RemoteDB.ApplyChanges();
        }
    }
}
