﻿using System;
using System.Windows.Forms;
using System.Data;
using System.Drawing;
using Cybele.Thinfinity;

namespace fishfact5
{
    public partial class FormMain : Form
    {
        private IJSObject mRemoteDB = null;
        private IJSObject mRemoteLayout = null;

        public FormMain() {
            InitializeComponent();

            // Creates "layout" JSObject
            mRemoteLayout = new JSObject("layout");

            // Adds a "bingSearch" property
            mRemoteLayout.Properties.Add("bingSearch")
                .OnGet(new JSBinding(          // Adds a "getter" to the returned IJSProperty
                // This anonymous procedure do the actual get
                delegate(IJSObject Parent, IJSProperty Prop) {
                    Prop.AsBool = chkBingSearch.Checked;
                }));

            // Adds a "windowVisible" property
            mRemoteLayout.Properties.Add("windowVisible")
                .OnGet(new JSBinding(         // Adds a "getter" to the returned IJSProperty
                // This anonymous procedure do the actual get
                delegate(IJSObject Parent, IJSProperty Prop) {
                    Prop.AsBool = Visible;
                }))
                .OnSet(new JSBinding(        // Adds a "setter" to the returned IJSProperty
                // This anonymous procedure do the actual set
                delegate(IJSObject Parent, IJSProperty Prop) {
                    Visible = Boolean.Parse(Prop.AsString);
                    if (!Visible) {
                        rbDesktopMode.Checked = true;
                        chkBingSearch.Checked = true;
                    }
                }));

            // Properties needed to embed the Bing Search iframe
            // Adds a "windowId" property with the calculated div id containing this window
            mRemoteLayout.Properties.Add("windowId").AsString = String.Format("virtualui_canvas_{0}", Handle);

            // Adds a "bounds" property with the coordinates where to embed the iframe
            mRemoteLayout.Properties.Add("bounds")
                .OnGet(new JSBinding(        // Adds a "getter" to the returned IJSProperty
                // This anonymous procedure do the actual get
                delegate(IJSObject AObj, IJSProperty AProp) {
                    Point p = pnlDetail.PointToScreen(new Point(0, 0));
                    p.Offset(-Left, -Top);
                    // Returns a JSON object
                    AProp.AsJSON = "{" + String.Format("\"left\":{0},\"top\":{1},\"width\":{2},\"height\":{3}",
                                            p.X, p.Y, pnlDetail.Width, pnlDetail.Height) + "}";
                }));

            // Adds the "embed" event with 1 boolean argument
            mRemoteLayout.Events.Add("embed").AddArgument("value", IJSDataType.JSDT_BOOL);

            // Applies the object model and s}s it to the web browser
            mRemoteLayout.ApplyModel();
        }

        public IJSObject RemoteDB {
            get {
                if (mRemoteDB == null) {
                    // Creates the "db" JSObject
                    mRemoteDB = new JSObject("db");

                    var allowedTypes = new[] { typeof(String), typeof(Byte), typeof(Char), typeof(Decimal), typeof(Double) };

                    // Iterates over the dataset fields
                    foreach (DataColumn column in dbdemosDataSet.BIOLIFE.Columns) {
                        // Bypasses some data types
                        if (Array.IndexOf(allowedTypes, column.DataType) >= 0) {
                            // Creates the property with the name of the field.
                            mRemoteDB.Properties.Add(column.ColumnName)     // Returns a IJSProperty
                                .OnGet(new JSBinding(                       // Adds a "getter" to the returned IJSProperty
                                // This anonymous procedure do the actual get
                                    delegate(IJSObject AObj, IJSProperty AProp) {
                                        DataColumn col = dbdemosDataSet.BIOLIFE.Columns[AProp.Name];
                                        string value = dbdemosDataSet.BIOLIFE.Rows[bIOLIFEBindingSource.Position][col].ToString();

                                        if ((col.DataType == typeof(String)) || (col.DataType == typeof(Byte)) || (col.DataType == typeof(Char))) {
                                            AProp.AsString = value;
                                        }
                                        else {
                                            AProp.AsFloat = float.Parse(value);
                                        }
                                    }
                                ));
                        }
                    }

                    // Adds a method to command the dataset navigation from Javascript
                    mRemoteDB.Methods.Add("navigate")                   // Returns a IJSMethod
                      .AddArgument("target", IJSDataType.JSDT_STRING)   // Adds a string parameter.
                      .OnCall(new JSCallback(                           // Adds the callback
                        // this anonymous procedure to do the actual execution
                        delegate(IJSObject Parent, IJSMethod Method) {
                            bool RV = true;
                            string Target = Method.Arguments["target"].AsString;

                            if (Target == "first")
                                bIOLIFEBindingSource.MoveFirst();
                            else if (Target == "last")
                                bIOLIFEBindingSource.MoveLast();
                            else if (Target == "prev")
                                bIOLIFEBindingSource.MovePrevious();
                            else if (Target == "next")
                                bIOLIFEBindingSource.MoveNext();
                            else
                                RV = false;
                            Method.ReturnValue.AsBool = RV;
                        }))
                      .ReturnValue.DataType = IJSDataType.JSDT_BOOL;   // Sets the return type

                    // Applies the object model and s}s it to the web browser
                    mRemoteDB.ApplyModel();
                }
                return mRemoteDB;
            }
        }

        private void FormMain_Load(object sender, EventArgs e) {
            // TODO: This line of code loads data into the 'dbdemosDataSet.BIOLIFE' table. You can move, or remove it, as needed.
            this.bIOLIFETableAdapter.Fill(this.dbdemosDataSet.BIOLIFE);
        }

        private void bIOLIFEBindingSource_PositionChanged(object sender, EventArgs e) {
            // Calls all mRemoteDB's properties "getters"
            RemoteDB.ApplyChanges();
        }

        private void chkBingSearch_CheckedChanged(object sender, EventArgs e) {
            rbEmbedMode.Enabled = chkBingSearch.Checked;
            rbDesktopMode.Enabled = chkBingSearch.Checked;
            LayoutChanged(sender, e);
        }

        private void rbDesktopMode_CheckedChanged(object sender, EventArgs e) {
            if (rbDesktopMode.Checked) {
                // Fires the 'embed' event
                mRemoteLayout.Events["embed"]
                  .ArgumentAsBool("value", rbEmbedMode.Checked)
                  .Fire();
                LayoutChanged(sender, e);
            }
        }

        private void rbEmbedMode_CheckedChanged(object sender, EventArgs e) {
            if (rbEmbedMode.Checked) {
                // Fires the 'embed' event
                mRemoteLayout.Events["embed"]
                  .ArgumentAsBool("value", rbEmbedMode.Checked)
                  .Fire();
                LayoutChanged(sender, e);
            }
        }

        private void LayoutChanged(object sender, EventArgs e) {
            // Calls all mRemoteLayout's properties "getters"
            if (mRemoteLayout != null) {
                mRemoteLayout.ApplyChanges();
            }
        }
    }
}
