﻿Imports System.Windows.Forms
Imports System.IO
Imports fishfact4.Cybele.Thinfinity

Public Class FormMain
    Private mVirualUI As VirtualUI
    Private mRemoteDB As IJSObject = Nothing
    Private mRemoteLayout As IJSObject = Nothing

    Public Sub New()
        InitializeComponent()
        AppDomain.CurrentDomain.SetData("DataDirectory", Path.GetFullPath("..\\..\\..\\..\\data"))

        mVirualUI = New VirtualUI()
        mVirualUI.Start()

        ' Creates "layout" JSObject
        mRemoteLayout = New JSObject("layout")

        ' Adds a "bingSearch" property
        ' Adds a "getter" to the returned IJSProperty (.OnGet)
        ' The anonymous Sub do the actual get
        mRemoteLayout.Properties.Add("bingSearch") _
            .OnGet(New JSBinding( _
                  Sub(Parent As IJSObject, Prop As IJSProperty)
                      Prop.AsBool = chkBingSearch.Checked
                  End Sub))

        ' Adds a "windowVisible" property
        ' Adds a "getter" to the returned IJSProperty (.OnGet)
        ' The anonymous Sub do the actual get
        ' Adds a "setter" to the returned IJSProperty (.OnSet)
        ' The anonymous Sub do the actual set
        mRemoteLayout.Properties.Add("windowVisible") _
            .OnGet(New JSBinding( _
                      Sub(Parent As IJSObject, Prop As IJSProperty)
                          Prop.AsBool = Visible
                      End Sub)) _
            .OnSet(New JSBinding( _
                     Sub(Parent As IJSObject, Prop As IJSProperty)
                         Visible = Boolean.Parse(Prop.AsString)
                         If Not Visible Then
                             chkBingSearch.Checked = True
                         End If
                     End Sub))

        ' Applies the object model and sends it to the web browser
        mRemoteLayout.ApplyModel()
    End Sub

    Public ReadOnly Property RemoteDB() As IJSObject
        Get
            If mRemoteDB Is Nothing Then
                ' Creates the "db" JSObject
                mRemoteDB = New JSObject("db")

                Dim allowedTypes = New Type() {GetType(String), GetType(Byte), GetType(Char), GetType(Decimal), GetType(Double)}

                ' Iterates over the dataset fields
                For Each column As DataColumn In dbdemosDataSet.BIOLIFE.Columns
                    ' Bypasses some data types
                    If Array.IndexOf(allowedTypes, column.DataType) >= 0 Then
                        ' Creates the property with the name of the field.
                        ' Returns a IJSProperty
                        ' Adds a "getter" to the returned IJSProperty (.OnGet)
                        ' The anonymous sub do the actual get
                        mRemoteDB.Properties.Add(column.ColumnName).OnGet( _
                            New JSBinding( _
                                Sub(AObj As IJSObject, AProp As IJSProperty)
                                    Dim col As DataColumn = DbdemosDataSet.BIOLIFE.Columns(AProp.Name)
                                    Dim value As String = DbdemosDataSet.BIOLIFE.Rows(BIOLIFEBindingSource.Position)(col).ToString()

                                    If (col.DataType = GetType(String)) OrElse (col.DataType = GetType(Byte)) OrElse (col.DataType = GetType(Char)) Then
                                        AProp.AsString = value
                                    Else
                                        AProp.AsFloat = Single.Parse(value)
                                    End If
                                End Sub))
                    End If
                Next

                ' Adds a method to command the dataset navigation from Javascript ("navigate")
                ' Returns a IJSMethod
                ' Adds a string parameter ("target")
                ' Adds the callback (.OnCall)
                ' The anonymous Sub to do the actual execution
                mRemoteDB.Methods.Add("navigate") _
                    .AddArgument("target", IJSDataType.JSDT_STRING) _
                    .OnCall(New JSCallback( _
                            Sub(Parent As IJSObject, Method As IJSMethod)
                                Dim RV As Boolean = True
                                Dim Target As String = Method.Arguments("target").AsString
                                If Target = "first" Then
                                    BIOLIFEBindingSource.MoveFirst()
                                ElseIf Target = "last" Then
                                    BIOLIFEBindingSource.MoveLast()
                                ElseIf Target = "prev" Then
                                    BIOLIFEBindingSource.MovePrevious()
                                ElseIf Target = "next" Then
                                    BIOLIFEBindingSource.MoveNext()
                                Else
                                    RV = False
                                End If
                                Method.ReturnValue.AsBool = RV
                            End Sub)) _
                    .ReturnValue.DataType = IJSDataType.JSDT_BOOL  ' Sets the return type

                ' Applies the object model and sends it to the web browser
                mRemoteDB.ApplyModel()
            End If
            Return mRemoteDB
        End Get
    End Property

    Private Sub FormMain_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        Me.BIOLIFETableAdapter.Fill(Me.DbdemosDataSet.BIOLIFE)
    End Sub

    Private Sub BIOLIFEBindingSource_PositionChanged(sender As Object, e As EventArgs) Handles BIOLIFEBindingSource.PositionChanged
        ' Calls all RemoteDB's properties "getters"
        RemoteDB.ApplyChanges()
    End Sub

    Private Sub CheckBox1_CheckedChanged(sender As Object, e As EventArgs) Handles chkBingSearch.CheckedChanged
        ' Calls all RemoteLayout's properties "getters"
        If (mRemoteLayout IsNot Nothing) Then
            mRemoteLayout.ApplyChanges()
        End If
    End Sub
End Class
